.TH MDATE 1L "June 1995"
.UC 6
.SH NAME
mdate \- get the time of day from a remote host
.SH SYNOPSIS
.B mdate
[
.B \-a|-s
] [
.B \-c count
] [
.B \-d
] [
.B \-t tol
] [
.B \-w wait
] 
.I host ...
.SH DESCRIPTION
.I Mdate
requests the time of day from
.I host,
which may be a host name or an IP address.
If more than one
.I host
is specified, they are queried in order until a response is received.
It calculates the time of day on the
.I host
and the estimated time difference between remote and local host.
A positive difference means the remote host is ahead of the local host.
.PP
The superuser can set the local time of day to agree with the time
on the remote host to within a small error.

.SH OPTIONS
.PP
.TP
.B \-a|-s
If the caller is the super-user and the time
difference is large enough, then
.I mdate
will use
.IR adjtime (2)
or
.IR settimeofday (2)
respectively to synchronise the local host's time with
the remote host. The
.I -a
flag is only allowed if
.IR adjtime (2) 
is actually available.
.TP
.BI \-c " count"
Repeat the call 
.I count 
times.
.TP
.BI \-d
Print verbose output for debugging. More 
.BI \-d
flags increase the verbosity.
.TP
.BI \-t " tol"
Change the tolerance to 
.I tol
quanta (default=3).
.TP
.BI \-w " wait"
Wait for 
.I n
seconds before calling the server, where
.I n
is a pseudo-random number in the range
.I 0 < n < wait

.SH "USAGE"

.I Mdate
is intended to be used as a simple device for keeping several machine 
clocks in sync over a small size local area network. It is based on 
the older programs
.I rdate
and 
.I in.timed .
The most important change is that 
.I mdate 
uses ICMP timestamps. This makes it far more accurate than 
.I rdate.
Over short distances (say, within a building)
.I mdate
can synchronise to within about 
10 milliseconds, which should be close enough for most purposes. 
The  disadvantage is that you have to be super-user to use
.I mdate
even just for testing.
.PP
Before starting to compile
.I mdate,
the installer should determine the 
.I quantum,
which is the smallest time interval that the local machine can measure. 
I have provided a supplementary program called
.I tick
to help with this. You should then edit the source file 
.I site.h
to define 
.I quantum
correctly.	
A time difference is deemed to be significant if it exceeds 
.I tolerance " * " quantum
and 
.I mdate
will not alter the local time if the estimated difference is
less than this amount. The
.I \-t
flag changes the 
.I tolerance
(default=3).
.PP
The 
.I \-c
flag makes 
.I mdate
call the remote host
.I count
times. This has 2 purposes. It can be used to probe the system's behaviour:
.I " mdate -d -d -c10 host "
This prints a small table. Also ICMP is not a reliable protocol, so if
some calls fail to get through, you can increase the chance of success 
by making repeated calls. As the distances get larger, the round-trip
times get larger and more variable. By repeating the call you have a much 
better chance of getting an accurate value.
.I mdate 
does not attempt any statistical analysis of the resulting data. It
merely accepts the time difference that goes with the shortest trip time.
.PP
By default,
.I mdate
prints nothing except error messages. It is intended to be called 
automatically by cron and (on our machines) cron seems to get 
confused by output.
The 
.I \-d
flag makes
.I mdate
print a line indicating success or failure. A 2nd and 3rd
.I \-d
increase the verbosity.
.PP
To keep a log, use just one
.I -d
and pipe the output to a file or to
.I logger 
(or 
.I syslog ).
It is advisable to do this at least for the first few days: there may 
be bugs; also the log will help to estimate your clock's rate of drift.
.PP
.I mdate
is intended as a lightweight mechanism for keeping time in a small
local network. The normal setup is to name one machine as local server. 
This gets its time from a national timeserver: you should first ask 
permission from that server's manager. Then all the other local 
machines would do something like:
.I mdate -c5 -w100 local-server
and you would use
.I cron
to make this happen at regular intervals.
.PP
If the clients all start pestering the server at once, there is 
a danger that the server
or net or both will overload. The flag
.I \-w wait
spreads the load. If this flag is present,
.I mdate 
will wait for a pseudo-random number of seconds in the range 0 ..
.I wait .
We suggest a 
.I wait
about 5 times the number of clients.  
.PP
With these additional features,
.I mdate
should be able to keep time to within about 0.001 sec for local
networks of up to around 100 machines. If you 
have more, you should either arrange a hierarchy of timeservers or 
install a more elephantine protocol such as xntp.
.PP
The multiple-host feature can be used to provide backups if one server is
down. Suppose there were 2 prospective local servers  
.IR A " and " B
and a national server 
.I N .
Then
.I A
might do
.I mdate (args) N
(say, at 1AM) and
.I B
might do
.I mdate (args) A N
at 2AM and the clients would do
.I mdate (args) A B
at 3AM. 

.SH "DIAGNOSTICS"
.I Mdate
checks the return status of system functions and uses
.I perror
to print error messages (which are usually not very informative). Errors
specific to 
.I Mdate
include the following;
.TP 
packet too short.
means we received something that looked like a truncated 
ICMP packet.
.TP 
huge difference.
A huge difference is a time difference >
about 2 hours. This is almost certainly caused by a bug in
.I Mdate ,
or maybe the server has gone mad.
If any machine is really that far out, set its date manually.
.TP 
time unchanged.
This should be either because the time diff was too small to be 
significant
or because the caller did not specify either
.IR -a " or " -s .
.PP
Exit status is 0 on success, otherwise 1.
.SH DEFECTS
Since the ICMP timer wraps around at midnight you may get a
"huge difference" error if you run mdate shortly before or after. 
You should try very hard to avoid using mdate near midnight.
Note this is midnight GMT not local time. 
.PP
Some machines do not
respond to ICMP time requests; they cannot act as servers but 
you might be able to run mdate on them. 
.PP
You have some protection against servers crashing (use multiple 
hosts); some protection against servers going wild (the huge 
difference check) but no protection at all against servers that drift.
.PP
Mysterious loss of accuracy. If 
.I mdate
measured a round-trip time of 3 msec, this ought to mean that the 
subsequent 
.I settimeofday
is that accurate. 
.SH AUTHOR
R.M.Damerell, Maths Dept, Royal Holloway College, Egham, Surrey, UK.
.PP
Program uses parts of rdate.c (by Dartmouth College) and ping.c (Mike
Muuss, US Army Ballistic Research Lab modified by Berkeley). Their
copyright notices are reproduced in full in the source code. Program 
is offered free of charge, "AS IS" with absolutely no guarantees.
.PP
Bug reports etc to RMD at this address: uhah208@ma.rhbnc.ac.uk (but
no promise to fix).
.SH "SEE ALSO"
adjtime(2), gettimeofday(2), settimeofday(2), tick(1), utimed(1),
